unit uActorNodal;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  uCosa, uGlobs, uActores, uFechas, unodos, uCosaConNombre, uFichasLPD,
  xMatDefs, uauxiliares;

type
  TActorNodal = class(TActor)
    function ReferenciaAlNodo(nodo: TNodo): boolean; virtual; abstract;
  end;

  TActorUniNodal = class(TActorNodal)
  public
    Nodo: TNodo;
    P: TDAOfNReal; // Inyectada como generador al nodo


    barras_flucar_: TDAOfNint;
    codigos_flucar_: TDAOfString;

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades: TFichasLPD; Nodo: TNodo);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    function ReferenciaAlNodo(nodo: TNodo): boolean; override;
    procedure PubliVars; override;
    procedure PrepararMemoria(globs: TGlobs); override;
    procedure Free; override;
  end;

  TActorBiNodal = class(TActorNodal)
  public
    NodoA, NodoB: TNodo;
    // resultados del despacho
    P_NodoA, P_NodoB: TDAOfNReal; //[MW] Potencia inyectada como generador en cada Nodo.

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades: TFichasLPD; NodoA, NodoB: TNodo);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    function ReferenciaAlNodo(nodo: TNodo): boolean; override;
    procedure PrepararMemoria(globs: TGlobs); override;
    procedure Free; override;
  end;

procedure AlInicio;
procedure ALFinal;

implementation

//--------------------------
// Mtodos de TActorUniNodal
//==========================

constructor TActorUniNodal.Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
  lpdUnidades: TFichasLPD; Nodo: TNodo);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades);
  self.Nodo := Nodo;
  barras_flucar_ := nil;
  codigos_flucar_ := nil;
end;

constructor TActorUniNodal.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rdReferencia('nodo', TCosa(nodo), Self);
  if f.Version < 49 then
  begin
    barras_flucar_ := nil;
    codigos_flucar_ := nil;
  end
  else
  begin
    f.rd('barras_flucar', barras_flucar_);
    f.rd('codigos_flucar', codigos_flucar_);
  end;
  f.EjecutarLectura;


  if f.Version >= 49 then
    clear_vacias(codigos_flucar_);

  self.Nodo := nil;
end;

procedure TActorUniNodal.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wrReferencia('nodo', Nodo);
  f.wr('barras_flucar', barras_flucar_);
  f.wr('codigos_flucar', codigos_flucar_);
end;

function TActorUniNodal.referenciaAlNodo(nodo: TNodo): boolean;
begin
  Result := self.nodo = nodo;
end;

procedure TActorUniNodal.PubliVars;
begin
  inherited PubliVars;
  PublicarVariableVR('P', '[MW]', 6, 2, P, True, True);
end;

procedure TActorUniNodal.PrepararMemoria(globs: TGlobs);
begin
  inherited PrepararMemoria(globs);
  setlength(P, globs.NPostes);
end;


procedure TActorUniNodal.Free;
begin
  setlength(P, 0);
  inherited Free;
end;

//-------------------------
// Mtodos de TActorBiNodal
//=========================

constructor TActorBiNodal.Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
  lpdUnidades: TFichasLPD; NodoA, NodoB: TNodo);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades);
  self.NodoA := NodoA;
  self.NodoB := NodoB;
end;

constructor TActorBiNodal.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rdReferencia('nodoA', TCosa(NodoA), Self);
  f.rdReferencia('nodoB', TCosa(NodoB), Self);
  f.EjecutarLectura;
end;

procedure TActorBiNodal.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wrReferencia('nodoA', NodoA);
  f.wrReferencia('nodoB', NodoB);
end;

function TActorBiNodal.referenciaAlNodo(nodo: TNodo): boolean;
begin
  Result := (nodoA = nodo) or (nodoB = nodo);
end;

procedure TActorBiNodal.prepararMemoria(globs: TGlobs);
begin
  inherited prepararMemoria(globs);
  setlength(P_NodoA, globs.NPostes);
  setlength(P_NodoB, globs.NPostes);
end;

procedure TActorBiNodal.Free;
begin
  setlength(P_NodoA, 0);
  setlength(P_NodoB, 0);
  inherited Free;
end;

procedure AlInicio;
begin
  //  registrarClaseDeCosa(TActorUniNodal.ClassName, TActorUniNodal);
  //  registrarClaseDeCosa(TActorBiNodal.ClassName, TActorBiNodal);

end;

procedure AlFinal;
begin
end;

end.
