unit uBaseMercadoSpot;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  uComercioInternacional, uFichasLPD, uFuentesAleatorias, xMatDefs,
  uFechas, uNodos,
  uCosa, uGlobs, uSimplex;

type
  TBaseMercadoSpot = class(TComercioInternacional)
  protected
    numeroBorne: integer;
  public
    fuente: TFuenteAleatoria;  //Se usa para calcular el costo variable
    nombreBorne: string;
    costos: TDAofNReal;
    //costos[iposte]:= P[iposte] * cv * globs.durpos[iposte];
    lpd: TFichasLPD;

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades, lpd: TFichasLPD; Nodo: TNodo; fuente: TFuenteAleatoria;
  nombreBorne: string); virtual;
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;

    class function TipoFichaLPD: TClaseDeFichaLPD; virtual; abstract;
    procedure PrepararMemoria(globs: TGlobs); override;

    procedure Free; override;
  end;

  TClaseDeMercadoSpot = class of TBaseMercadoSpot;

implementation

constructor TBaseMercadoSpot.Create( capa: integer; nombre: string; nacimiento, muerte: TFecha;
  lpdUnidades, lpd: TFichasLPD; Nodo: TNodo; fuente: TFuenteAleatoria;
  nombreBorne: string);
begin
  inherited Create( capa, nombre, nacimiento, muerte, lpdUnidades, Nodo);
  self.lpd := lpd;
  self.fuente := fuente;
  self.nombreBorne := nombreBorne;
end;

constructor TBaseMercadoSpot.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rdReferencia('fuente', TCosa(Fuente), Self);
  f.rd('nombreBorne', nombreBorne);
  f.rd('lpd', TCosa(lpd));
  f.EjecutarLectura;
  lpd.Propietario := self;
end;

procedure TBaseMercadoSpot.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wrReferencia('fuente', fuente);
  f.wr('nombreBorne', nombreBorne);
  f.wr('lpd', TCosa(lpd));
end;

procedure TBaseMercadoSpot.PrepararMemoria(globs: TGlobs);
begin
  inherited PrepararMemoria(globs);
  SetLength(costos, globs.NPostes);
  numeroBorne := fuente.idBorne(nombreBorne);
end;

procedure TBaseMercadoSpot.Free;
begin
  if length(costos) <> 0 then
    SetLength(costos, 0);
  if lpd <> nil then
    lpd.Free;
  inherited Free;
end;


end.

