unit ugsimple_bicombustible;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  Math,
  SysUtils, Classes, xmatdefs, uGTer, uNodos,
  uglobs,
  usimplex,
  ufichasLPD,
  ufechas,
  ucosa,
  uconstantesSimSEE,
  uFuentesAleatorias,
  unodocombustible;

resourcestring
  rsGeneradorSimpleBiCombustible = 'Generador simple bicombustible';

type
  TFichaGSimple_BiCombustible = class(TFichaLPD)
  public
    PagoPorDisponibilidad_USD_MWh: NReal; // [USD/MWh] Pago por Potencia
    PagoPorEnergia_USD_MWh: NReal; // [USD/MWh] Pago por Potencia
    PMax: NReal; // [MW] Potencia Máxima Por maquina
    QAMax, QBMax: NReal; // Máximos caudales de cada combustible
    disp: NReal; // disponibilidad (fortuita)
    tRepHoras: NReal;

    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      PMax, QAMax, QBMax: NReal; disp: NReal; tRepHoras: NReal;
      PagoPorDisponibilidad_USD_MWh: NReal; PagoPorEnergia_USD_MWh: NReal);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    function infoAd_: string; override;
    procedure Free; override;
  end;





  // TGSimple_Bicombustible es un generador sin minimos tecnicos ni tiempos ni costos
  // de arranque y parada
  TGSimple_BiCombustible = class(TGTer)
  public
    pa: TFichaGSimple_BiCombustible;

    nodoCombA: TNodoCombustible;
    nodoCombB: TNodoCombustible;

    NMaquinasDespachadas: TDAOfNInt;    // cantidad de máquinas despachadas por poste o por paso ( Acople )

    consumos: TDAofNReal;              // consumos por máquina por poste

    maxNMaquinasDespachadas: integer; // máximo del vector anterior

    cve: NReal; // costo por la energía adicional a CV
    efi_a: NReal;
    efi_b: NReal;

    P_opmax: NReal;



    qa: TDAOfNReal;
    qb: TDAOfNReal;

    alfa: TDAOfNReal;
    // resultado de los sorteos de disponibilidad y del mantenimiento programado
    NMaquinasDisponibles: integer;

    PMaxDisponible: NReal;
    QAMaxOperativo, QBMaxOperativo: NReal;

    //Cuanta potencia despacho en cada hora el generador en promedio
    potMedia_despachada: NReal;

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades, lpd: TFichasLPD; nodo: TNodo;
      nodoCombA, nodoCombB: TNodoCombustible;
      TonCO2xMWh: NReal;
      LowCostMustRun, CleanDevelopmentMechanism: boolean);

    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    procedure PrepararMemoria(globs: TGlobs); override;
    procedure RegistrarParametrosDinamicos; override;

    function PotenciaFirme: NReal; override;

    function InfoAd_: string; override;
    class function DescClase: string; override;

    procedure SorteosDelPaso(sortear: boolean); override;
    procedure CambioFichaPD; override;
    procedure PrepararPaso_ps; override;
    procedure Sim_Paso_Fin; override;

    procedure opt_nvers(var ivar, ivae, ires: integer); override;

    procedure opt_cargue(s: TSimplex); override;
    procedure opt_fijarRestriccionesDeCaja(s: TSimplex); override;
    procedure opt_leerSolucion(s: TSimplex); override;


{$IFDEF SPXCONLOG}
    procedure spx_NombrarVariables(s: TSimplex); override;
{$ENDIF}
    function getNombreVar(ivar: integer; var nombre: string): boolean; override;
    function getNombreRes(ires: integer; var nombre: string): boolean; override;

    procedure Free; override;

    procedure PubliVars; override;
    procedure dump_Variables(var f: TextFile; charIndentacion: char); override;

    class function TipoFichaLPD: TClaseDeFichaLPD; override;

    function get_pa_FD( kTipoUnidad: integer ): NReal; override;
    function get_pa_TMR( kTipoUnidad: integer ): NReal; override;

  end;


procedure AlInicio;
procedure AlFinal;

implementation


//---------------------------------------
// Métodos de TFichaGSimple_BiCombustible
//=======================================
constructor TFichaGSimple_BiCombustible.Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      PMax, QAMax, QBMax: NReal; disp: NReal; tRepHoras: NReal;
      PagoPorDisponibilidad_USD_MWh: NReal; PagoPorEnergia_USD_MWh: NReal);

begin
  inherited Create(capa, fecha, periodicidad);
  self.PMax := PMax;
  self.QAMax:= QAMax;
  self.QBMax:= QBMax;
  self.disp := disp;
  self.tRepHoras := tRepHoras;
  self.PagoPorDisponibilidad_USD_MWh := PagoPorDisponibilidad_USD_MWh;
  self.PagoPorEnergia_USD_MWh := PagoPorEnergia_USD_MWh;
end;

constructor TFichaGSimple_BiCombustible.Create_ReadFromText(f: TArchiTexto);
begin

    inherited Create_ReadFromText(f);
    f.IniciarLecturaRetrasada;
    f.rd('PMax', PMax);
    f.rd('disp', disp);
    f.rd('tRepHoras', tRepHoras);
    f.rd('PagoPorDisponibilidad_USD_MWh', PagoPorDisponibilidad_USD_MWh);
    f.rd('PagoPorEnergia_USD_MWh', PagoPorEnergia_USD_MWh);

    f.EjecutarLectura;

end;

procedure TFichaGSimple_BiCombustible.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('PMax', PMax, uconstantesSimSEE.CF_PRECISION, uconstantesSimSEE.CF_DECIMALES);
  f.wr('disp', disp, uconstantesSimSEE.CF_PRECISION, uconstantesSimSEE.CF_DECIMALESPU);
  f.wr('tRepHoras', tRepHoras, uconstantesSimSEE.CF_PRECISION,
    uconstantesSimSEE.CF_DECIMALESPU);
  f.wr('PagoPorDisponibilidad_USD_MWh', PagoPorDisponibilidad_USD_MWh);
  f.wr('PagoPorEnergia_USD_MWh', PagoPorEnergia_USD_MWh);
end;

function TFichaGSimple_BiCombustible.infoAd_: string;
begin
  Result := 'PMax= ' + FloatToStrF(PMax, ffGeneral, 10, 1) + ' MW, ' +
    'fdisp= ' + FloatToStrF(disp, ffGeneral, 10, 2) + ' p.u., ' +
    'tRep= ' + FloatToStrF(tRepHoras, ffGeneral, 10, 1) + 'h';
end;

procedure TFichaGSimple_BiCombustible.Free;
begin
  inherited Free;
end;

//------------------------
// Métodos de TGSimple_BiCombustible
//========================
procedure TGSimple_BiCombustible.dump_Variables(var f: TextFile; charIndentacion: char);
begin
  inherited dump_Variables(f, charIndentacion);

  writeln(f, charIndentacion, 'PMax[MW]= ', FloatToStrF(pa.PMax, ffFixed, 10, 3));
  writeln(f, charIndentacion, 'PMaxDisponible[MW]= ',
    FloatToStrF(PMaxDisponible, ffFixed, 10, 3));

  writeln(f, charIndentacion, 'NMaquinasDisponibles= ', NMaquinasDisponibles);

  writeln(f);
end;

class function TGSimple_BiCombustible.TipoFichaLPD: TClaseDeFichaLPD;
begin
  Result := TFichaGSimple_BiCombustible;
end;

constructor TGSimple_BiCombustible.Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
  lpdUnidades, lpd: TFichasLPD; nodo: TNodo;
  nodoCombA, nodoCombB: TNodoCombustible;
  TonCO2xMWh: NReal;
  LowCostMustRun, CleanDevelopmentMechanism: boolean);
begin
  inherited Create(capa, nombre, nacimiento, muerte,
  lpdUnidades, lpd, nodo,
    TonCO2xMWh, LowCostMustRun, CleanDevelopmentMechanism);
  P := nil;
  qa := nil;
  qb := nil;
  NMaquinasDespachadas := nil;
  self.nodoCombA:= nodoCombA;
  self.nodoCombB:= nodoCombB;
end;

constructor TGSimple_BiCombustible.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.rdReferencia( 'NodoCombA', TCosa( NodoCombA ), self );
  f.rdReferencia( 'NodoCombB', TCosa( NodoCombB ), self );
  P := nil;
  qa := nil;
  qb := nil;
  NMaquinasDespachadas := nil;
  pa := nil;

end;

procedure TGSimple_BiCombustible.prepararMemoria(globs: TGlobs);
begin
  inherited prepararMemoria(globs);
  setlength(qa, globs.NPostes);
  setlength(qb, globs.NPostes);
  setlength(alfa, globs.NPostes);
  SetLength(NMaquinasDespachadas, globs.NPostes);
end;

procedure TGSimple_BiCombustible.registrarParametrosDinamicos;
begin
  inherited registrarParametrosDinamicos;

end;

function TGSimple_BiCombustible.potenciaFirme: NReal;
begin
  Result := (paUnidades.nUnidades[0]) * pa.PMax * pa.disp;
end;

function TGSimple_BiCombustible.infoAd_: string;
begin
  Result := '';
end;

class function TGSimple_BiCombustible.DescClase: string;
begin
  Result := rsGeneradorSimpleBiCombustible;
end;


function TGSimple_BiCombustible.get_pa_FD( kTipoUnidad: integer ): NReal;
begin
  result:= pa.disp;
end;

function TGSimple_BiCombustible.get_pa_TMR( kTipoUnidad: integer ): NReal;
begin
  result:= pa.tRepHoras;
end;

procedure TGSimple_BiCombustible.SorteosDelPaso(sortear: boolean);
begin
  if hayForzamientos or globs.ObligarDisponibilidad_1_ then
  begin
    NMaquinasDisponibles := paUnidades.nUnidades[0];   //supongo que nUnidades[0] tiene el total de maquinas de la central
    PMaxDisponible := pa.PMax * NMaquinasDisponibles;
  end
  else if sortear then
  begin
    ActualizarProbabilidadesReparacionYRotura_(pa.disp, pa.tRepHoras);
    NMaquinasDisponibles := Sorteos_RepRotUnidades;
    PMaxDisponible := pa.PMax * NMaquinasDisponibles;  //si hay que sortear hace el sorteo y la potencia total
  end                                                  //queda afectada por la cantidad de maquinas disponibles

  else
  begin
    NMaquinasDisponibles := paUnidades.nUnidades[0];            //si no hay que sortear deja todas las maquinas disponibles
    PMaxDisponible := pa.PMax * pa.disp * NMaquinasDisponibles; //pero afecta a la potencia disponible por la disponibilidad
  end;
end;

procedure TGSimple_BiCombustible.prepararPaso_ps;
{$IFDEF CALC_MMEE}
var
  k: integer;
{$ENDIF}
begin


  {$IFDEF CALC_MMEE}
  for k := 0 to globs.NPOstes - 1 do
    mmee_cv_paraPrecioSpot[k] := cv;
  {$ENDIF}

end;

procedure TGSimple_BiCombustible.Sim_Paso_Fin;
var
  iposte: integer;
  ePoste: NReal;
begin
  potMedia_despachada := 0;


  if NMaquinasDisponibles = 0 then
  begin
  {$IFDEF CALC_MMEE}
    vclear(mmee_IngresosSpot_USD);
    vclear(mmee_CargosAlSeguimientoDeLaDemanda_USD);
  {$ENDIF}
  end
  else
  begin
    for iposte := 0 to high(P) do
    begin
      ePoste := P[iposte] * globs.durpos[iposte];
      potMedia_despachada := potMedia_despachada + ePoste;

      {$IFDEF CALC_MMEE}
      if (mmee_cv_paraPrecioSpot[iposte] > globs.mmee_topePrecioSpot) then
      begin
        mmee_CargosAlSeguimientoDeLaDemanda_USD[iposte] :=
          ePoste * (mmee_cv_paraPrecioSpot[iposte] - globs.mmee_topePrecioSpot);
        mmee_IngresosSpot_USD[iposte] := ePoste * globs.mmee_topPrecioSpot;
      end
      else
      begin
        mmee_CargosAlSeguimientoDeLaDemanda_USD[iposte] := 0;
        mmee_IngresosSpot_USD[iposte] := ePoste * globs.mmee_PrecioSpot[iposte];
      end;
      {$ENDIF}
    end;
    potMedia_despachada := potMedia_despachada * globs.invHorasDelPaso;
  end;
end;

procedure TGSimple_BiCombustible.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wrReferencia( 'NodoCombA', NodoCombA  );
  f.wrReferencia( 'NodoCombB', NodoCombB  );
end;

procedure TGSimple_BiCombustible.opt_nvers(var ivar, ivae, ires: integer);
begin
  if NMaquinasDisponibles = 0 then
    exit; // si no hay máquinas no juego

  Self.ivar := ivar;
  ivar := ivar + 3*globs.NPostes; //qa, qb, alfa
  Self.ires := ires;
  ires:=ires + 2*globs.NPostes; //una restriccion por caudal de combustible qa<qamax*alfa y qb<qbmax*(1-alfa)

end;


procedure TGSimple_BiCombustible.CambioFichaPD;
//donde uso las potencias de operacion? en el simplex impongo restriccion
//de caudales pero podria ser que la Pop este limitada por la Pmax y en ese caso el caudal que obtengo me dara una P mayor
//que la que puedo dar Pop=Pmax.
begin
  QAMaxOperativo:= Math.Min(pa.PMax / ( efi_a * nodoCombA.combustible.pa.PCI ) , pa.qAMax);
  QBMaxOperativo:= Math.Min(pa.PMax / (efi_b*nodoCombB.combustible.pa.PCI ) , pa.QBMax);
end;

procedure TGSimple_BiCombustible.opt_cargue(s: TSimplex);
var
  inodores,inodoACombres,inodoBCombres: integer;
  iposte: integer;
  pci_a:NReal;
  pci_b:NReal;

  begin

  pci_a:=nodoCombA.combustible.pa.PCI;
  pci_b:=nodoCombB.combustible.pa.PCI;

  if NMaquinasDisponibles = 0 then
    exit; // si no hay máquinas no juego

{$IFDEF SPXCONLOG}
  spx_NombrarVariables(s);
{$ENDIF}

  inodores := nodo.ires; //indice de la restricion de nodo electrico
  inodoACombres := nodoCombA.ires; //indice de la restricion del nodo A de combustible
  inodoBCombres := nodoCombB.ires; //indice de la restricion del nodo B de combustible

  for iposte := 0 to globs.NPostes - 1 do
  begin
    // cargamos la potencia en la restricción de balance de potencia del nodo
    s.pon_e(inodores + iposte, ivar + iposte*3, efi_a*pci_a);
    s.pon_e(inodores + iposte, ivar + 1 + iposte*3, efi_b*pci_b);

    // cargamos los caudales en la restricción de balance de caudal de los nodos
    s.pon_e(inodoACombres + iposte, ivar + iposte*3, 1);
    s.pon_e(inodoBCombres + iposte, ivar + 1 + iposte*3, 1);

    // restricciones de los caudales qa y qb
    s.pon_e(ires + 2*iposte, ivar + iposte*3, -1);
    s.pon_e(ires + 2*iposte, ivar + 2 + iposte*3, QAMaxOperativo );
    s.pon_e(ires + 1 + 2*iposte, ivar + 1 + iposte*3, -1);
    s.pon_e(ires + 1 + 2*iposte, ivar + 2 + iposte*3, -QBMaxOperativo );
    s.pon_e(ires + 1 + 2*iposte, s.nc, QBMaxOperativo);


  end;

end;

procedure TGSimple_BiCombustible.opt_fijarRestriccionesDeCaja(s: TSimplex);
var
  iposte: integer;
begin
  if NMaquinasDisponibles = 0 then
    exit;

  // obligamos a que alfa se encuentre entre 0 y 1 en todos lso postes
  // y que los caudales sean menores que qamax y qbmax

  for iposte := 0 to globs.NPostes - 1 do
      s.cota_sup_set(ivar + 2 + 3*iposte, 1);
      s.cota_sup_set(ivar+ 3*iposte , qAMaxOperativo);
      s.cota_sup_set(ivar + 1 + 3*iposte , qBMaxOperativo );
end;

procedure TGSimple_BiCombustible.opt_leerSolucion(s: TSimplex);
var
  iposte: integer;
  mc, me, meAcum,pci_a,pci_b: NReal;

begin


  pci_a:=nodoCombA.combustible.pa.PCI;
  pci_b:=nodoCombB.combustible.pa.PCI;

  maxNMaquinasDespachadas := 0;

  if NMaquinasDisponibles = 0 then
  begin
    vclear(P);
    vclear(qa);
    vclear(qb);
    vclear(alfa);

    vclear(NMaquinasDespachadas);
    Ingreso_PorDisponibilidad := 0;
    Ingreso_PorEnergia := 0;
    costoDirectoDelPaso := 0;
    exit;
  end;

  Ingreso_PorDisponibilidad := self.PMaxDisponible * globs.HorasDelPaso *    //ingreso por potencia
    pa.PagoPorDisponibilidad_USD_MWh;
  CostoDirectoDelPaso := Ingreso_PorDisponibilidad;

  // recuperamos los valores de qa y qb despachadas
  meAcum := 0;
  for iposte := 0 to globs.NPostes - 1 do
  begin
    qa[iposte] := s.xval(ivar + iposte*3);
    qb[iposte] := s.xval(ivar + iposte*3 + 1);
    alfa[iposte] := s.xval(ivar + iposte*3 + 2);

    P[iposte]:= qa[iposte]*pci_a*efi_a*alfa[iposte] + qb[iposte]*pci_b*efi_b*(1-alfa[iposte]);

    if P[iposte] > GTER_PRECISIONPOTENCIANMAQS then
    begin
      NMaquinasDespachadas[iposte] :=
        Math.Ceil((P[iposte] - GTER_PRECISIONPOTENCIANMAQS) / pa.PMax);
      me := P[iposte] * globs.DurPos[iposte];


      meAcum := meAcum + me;
      if NMaquinasDespachadas[iposte] > maxNMaquinasDespachadas then
        maxNMaquinasDespachadas := NMaquinasDespachadas[iposte];
    end
    else
    begin
      NMaquinasDespachadas[iposte] := 0;
    end;
  end;

  Ingreso_PorEnergia := meAcum * pa.PagoPorEnergia_USD_MWh;

  CostoDirectoDelPaso := CostoDirectoDelPaso + Ingreso_PorEnergia;

end;


{$IFDEF SPXCONLOG}
procedure TGSimple_BiCombustible.spx_NombrarVariables(s: TSimplex);
var
  iposte: integer;
begin
  if NMaquinasDisponibles = 0 then
    exit;
  for iposte := 0 to globs.NPostes - 1 do
      s.set_NombreVar(ivar + iposte*3, Nombre + '_Qa[m3]' + IntToStr(iposte + 1));
      s.set_NombreVar(ivar + iposte*3+1, Nombre + '_Qb[m3]' + IntToStr(iposte + 1));
      s.set_NombreVar(ivar + iposte*3+2, Nombre + '_alfa' + IntToStr(iposte + 1));

end;

{$ENDIF}

function TGSimple_BiCombustible.getNombreVar(ivar: integer; var nombre: string): boolean;
begin
  if NMaquinasDisponibles = 0 then
    Result := False
  else
  if (ivar >= self.ivar) and (ivar < self.ivar + globs.NPostes) then
  begin
    nombre := self.Nombre + '_P[MW]' + IntToStr(ivar - self.ivar + 1);  //definir todas las ivar
    Result := True;
  end
  else
    Result := False;
end;

function TGSimple_BiCombustible.getNombreRes(ires: integer; var nombre: string): boolean;
begin
  if NMaquinasDisponibles = 0 then
    Result := False
  else
  if ires = self.ires then
  begin
    nombre := self.nombre + '_res-caudalCombA';
    Result := True;
  end;

  if ires = self.ires+1 then
  begin
    nombre := self.nombre + '_res-caudalCombB';
    Result := True;
  end

  else
    Result := False;
end;

procedure TGSimple_BiCombustible.PubliVars;
begin
  inherited PubliVars;
  PublicarVariableVI('NMaqsDespachadas', '-', NMaquinasDespachadas, True, True);

  {$IFDEF DECLARAR_VARIABLES_SIMRES_DEF}
  declararVarsPSimResPorDefectoIntercalandoPostes(['P', 'Costo', 'NMaqsDespachadas'],
    globs.NPostes);
  {$ENDIF}

  PublicarVariableNI('MaxNMaqsDespachadasEnElPaso', '-', maxNMaquinasDespachadas, False);
  PublicarVariableNI('NMaqsDisponibles', '-', NMaquinasDisponibles, False);
  PublicarVariableNR('PMáxDisponible', '[MW]', 6, 2, PMaxDisponible, False);
  PublicarVariableNR('PMediaDespachada', '[MW]', 6, 2, potMedia_despachada, False);

end;

procedure TGSimple_BiCombustible.Free;
begin
  setlength(NMaquinasDespachadas, 0);
  inherited Free;
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(TGSimple_BiCombustible.ClassName, TGSimple_BiCombustible);
  registrarClaseDeCosa(TFichaGSimple_BiCombustible.ClassName, TFichaGSimple_BiCombustible);
end;

procedure AlFinal;
begin
end;

end.
