// definicin de variables y operaciones estadsticas
unit uHistoVarsOps;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  sysutils, xmatdefs, uTSimRes, classes, uAuxiliares, uSimResGlobs;

type
  TClaseDeIndice = class of TVarIdxs;
  TVarIdxs= class
    idxs: TDAofNInt;
    nombreIndice: String;
    nombreActor, nombreVar: String;
    numSimRes: String;
    simRes: TResultadoSim;
    constructor Create( nombreIndice, nombreActor, nombreVar, numSimRes: String);
    procedure prepararse(simRes: TResultadoSim);
    procedure Free;
    function serialize: string; virtual;
  end;

  TDAOfTVarIdxs = array of TVarIdxs;

  TClaseDeCronVar = class of TCronVar;
  TCronVar= class
    public
      nombre: String;

      // ojo, el primer ndice es el paso de tiempo y el segundo la crnica
      v: TMatOfNReal;
      nCronicas, nPasos: integer;
      dtIni: TDateTime;
      DurPaso_horas: NReal;

      constructor Create( nombre: string );
      procedure inicializar(nCronicas, nPasos: integer; dtIni: TDateTime;
                            durPaso_Horas: NReal);

      function vpe(kPaso: Integer; probabildadExcedencia: NReal): NReal;
      procedure Free;
      function serialize: string; virtual;
  end;

  TDAOfCronVar = array of TCronVar;

  TClaseDeCronOper = class of TCronOper;
  TCronOper= class
    public
      res: TCronVar;

      //Resuelve los valores que tengan que ver con los ndices o el
      //encabezado de la simulacion
      procedure prepararse; virtual; abstract;
      procedure Evaluar; virtual; abstract;
      //Retorna un string con la descripcion de la operacion y su sintaxis
      class function ayudaCronOper: String; virtual; abstract;
      class function tipo: String; virtual; abstract;

      function referenciaIndice(indice: TVarIdxs): boolean; virtual; abstract;
      function parametrosIndice: String; virtual; abstract;

      // retorna TRUE si la cronVar est entre los parmetros o es resultado.
      function referenciaCronVar(cronVar: TCronVar): boolean; virtual;

      function resultados: String; virtual;
      function parametrosAdicionales: String; virtual;
      function serialize: string; virtual;
  end;
  TDAOfCronOper = array of TCronOper;

  TCronOperUnIndice= class(TCronOper)
    protected
      indices: TDAofNInt;
    public
      param1: TVarIdxs;
      function referenciaIndice(indice: TVarIdxs): boolean; override;
      function parametrosIndice: String; override;
  end;

  TCronOperDosIndices= class(TCronOperUnIndice)
    protected
      indices2: TDAofNInt;
    public
      param2: TVarIdxs;
      function referenciaIndice(indice: TVarIdxs): boolean; override;
      function parametrosIndice: String; override;
  end;

  TCronOperMultiIndice= class(TCronOper)
    protected
      indices: TMatOfNInt;
    public
      params: TDAOfTVarIdxs;
      function referenciaIndice(indice: TVarIdxs): boolean; override;
      function parametrosIndice: String; override;
  end;

  TCronOper_suma= class( TCronOperUnIndice )
    public
      constructor Create( res: TCronVar; param1: TVarIdxs );
      //Resuelve los valores que tengan que ver con los ndices o el
      //encabezado de la simulacion
      procedure prepararse; override;
      procedure Evaluar; override;
      class function ayudaCronOper: String; override;
      class function tipo: String; override;
      function serialize: string; override;
  end;

  TCronOper_sumaConSigno= class( TCronOperUnIndice )
    private
      sumaPos, sumaNeg: Double;
    public
      resNeg: TCronVar;

      constructor Create( resPos, resNeg: TCronVar; param1: TVarIdxs );
      //Resuelve los valores que tengan que ver con los ndices o el
      //encabezado de la simulacion
      procedure prepararse; override;
      procedure Evaluar; override;
      class function ayudaCronOper: String; override;
      class function tipo: String; override;
      function referenciaCronVar(cronVar: TCronVar): boolean; override;
      function resultados: String; override;
      function serialize: string; override;
  end;

  TCronOper_Combinar= class( TCronOperMultiIndice )
    public
      coefs: TDAofNReal;

      constructor Create( res: TCronVar; params: TDAOfTVarIdxs; coefs: TDAofNReal );
      //Resuelve los valores que tengan que ver con los ndices o el
      //encabezado de la simulacion
      procedure prepararse; override;
      procedure Evaluar; override;
      procedure Free;
      class function ayudaCronOper: String; override;
      class function tipo: String; override;
      function parametrosAdicionales: String; override;
      function serialize: string; override;
  end;

  TCronOper_promedio= class( TCronOperUnIndice )
    public
      constructor Create( res: TCronVar; param1: TVarIdxs );
      //Resuelve los valores que tengan que ver con los ndices o el
      //encabezado de la simulacion
      procedure prepararse; override;
      procedure Evaluar; override;
      class function ayudaCronOper: String; override;
      class function tipo: String; override;
      function serialize: string; override;
  end;

  TCronOper_sumaProductoConDurpos= class( TCronOperUnIndice )
    public
      constructor Create( res: TCronVar; param1: TVarIdxs );
      //Resuelve los valores que tengan que ver con los ndices o el
      //encabezado de la simulacion
      procedure prepararse; override;
      procedure Evaluar; override;
      class function ayudaCronOper: String; override;
      class function tipo: String; override;
      function serialize: string; override;
  end;

  TCronOper_sumaProductoConDurposHasta= class( TCronOperUnIndice )
    public
      kposteHasta: integer;

      constructor Create( res: TCronVar; param1: TVarIdxs; kposteHasta: integer );
      //Resuelve los valores que tengan que ver con los ndices o el
      //encabezado de la simulacion
      procedure prepararse; override;
      procedure Evaluar; override;
      class function ayudaCronOper: String; override;
      class function tipo: String; override;
      function parametrosAdicionales: String; override;
      function serialize: string; override;
  end;

  TCronOper_promedioPonderadoPorDurpos= class( TCronOperUnIndice )
    public
      constructor Create( res: TCronVar; param1: TVarIdxs );
      //Resuelve los valores que tengan que ver con los ndices o el
      //encabezado de la simulacion
      procedure prepararse; override;
      procedure Evaluar; override;
      class function ayudaCronOper: String; override;
      class function tipo: String; override;
      function serialize: string; override;
  end;

  TCronOper_filtrarCronica= class( TCronOperUnIndice )
    public
      kCronica: Integer;
      constructor Create( res: TCronVar; param1: TVarIdxs; kCronica: Integer);
      //Resuelve los valores que tengan que ver con los ndices o el
      //encabezado de la simulacion
      procedure prepararse; override;
      procedure Evaluar; override;
      class function ayudaCronOper: String; override;
      class function tipo: String; override;
      function parametrosAdicionales: String; override;
      function serialize: string; override;
  end;

// si no interesa el recorte poner nil
  TCronOper_sumaProductoConDurposTopeado= class( TCronOperUnIndice )
    private
      recorte: Double;
    public
      resRecorte: TCronVar;
      TopeDe1: NReal;

      constructor Create( resSP, resRecorte: TCronVar; param1: TVarIdxs; TopeDe1: NReal );
      //Resuelve los valores que tengan que ver con los ndices o el
      //encabezado de la simulacion
      procedure prepararse; override;
      procedure Evaluar; override;
      class function ayudaCronOper: String; override;
      class function tipo: String; override;
      function referenciaCronVar(cronVar: TCronVar): boolean; override;
      function resultados: String; override;
      function parametrosAdicionales: String; override;
      function serialize: string; override;

  end;

// si no interesa el recorte poner nil
  TCronOper_sumaDobleProductoConDurposTopeado= class( TCronOperDosIndices )
    private
      recorte: Double;
    public
      resRecorte: TCronVar;
      TopeDe2: NReal;

      constructor Create( resSP, resRecorte: TCronVar; param1, param2: TVarIdxs; TopeDe2: NReal );
      //Resuelve los valores que tengan que ver con los ndices o el
      //encabezado de la simulacion
      procedure prepararse; override;
      procedure Evaluar; override;
      class function ayudaCronOper: String; override;
      class function tipo: String; override;
      function referenciaCronVar(cronVar: TCronVar): boolean; override;
      function resultados: String; override;
      function parametrosAdicionales: String; override;
      function serialize: string; override;
  end;

  TCronOper_suma_m= class( TCronOperMultiIndice )
    public
      constructor Create( res: TCronVar; params: TDAOfTVarIdxs );
      //Resuelve los valores que tengan que ver con los ndices o el
      //encabezado de la simulacion
      procedure prepararse; override;
      procedure Evaluar; override;
      class function ayudaCronOper: String; override;
      class function tipo: String; override;
      procedure Free;
      function serialize: string; override;
  end;

  TCronOper_promedio_m= class( TCronOperMultiIndice )
    public
      constructor Create( res: TCronVar; params: TDAOfTVarIdxs );
      //Resuelve los valores que tengan que ver con los ndices o el
      //encabezado de la simulacion
      procedure prepararse; override;
      procedure Evaluar; override;
      class function ayudaCronOper: String; override;
      class function tipo: String; override;
      procedure Free;
      function serialize: string; override;
  end;

  TCronOper_sumaProductoConDurpos_m= class( TCronOperMultiIndice )
    public
      constructor Create( res: TCronVar; params: TDAOfTVarIdxs );
      //Resuelve los valores que tengan que ver con los ndices o el
      //encabezado de la simulacion
      procedure prepararse; override;
      procedure Evaluar; override;
      class function ayudaCronOper: String; override;
      class function tipo: String; override;
      procedure Free;
      function serialize: string; override;
  end;

  TCronOper_Maximo_m= class( TCronOperMultiIndice )
    public
      constructor Create( res: TCronVar; params: TDAOfTVarIdxs );
      //Resuelve los valores que tengan que ver con los ndices o el
      //encabezado de la simulacion
      procedure prepararse; override;
      procedure Evaluar; override;
      class function ayudaCronOper: String; override;
      class function tipo: String; override;
      procedure Free;
      function serialize: string; override;
  end;

implementation
uses
  uAuxiliaresEscrituraSimRes;


//-------------------
//Mtodos de TVarIdxs
//===================

constructor TVarIdxs.Create( nombreIndice, nombreActor, nombreVar, numSimRes: String);
begin
  inherited Create;
  self.nombreIndice:= nombreIndice;
  self.nombreActor:= nombreActor;
  self.nombreVar:= nombreVar;
  self.numSimRes:= numSimRes;
  self.simRes:= NIL;
  self.idxs:= NIL;
end;

procedure TVarIdxs.prepararse(simRes: TResultadoSim);
begin
  self.simRes:= simRes;
  idxs:= simRes.getIndices( nombreActor, nombreVar );
end;

procedure TVarIdxs.Free;
begin
  setlength( idxs, 0 );
  inherited Free;
end;

function TVarIdxs.serialize: string;
begin
    result:= encomille( nombreIndice ) + #9 +
          encomille( nombreActor ) + #9 +
          encomille( nombreVar ) + #9 +
          numSimRes;
end;


//-------------------
//Mtodos de TCronVar
//===================

constructor TCronVar.Create( nombre: string );
begin
  inherited Create;
  self.nombre:= nombre;
  nCronicas:= -1;
  nPasos:= -1;
  dtIni:= -1;
  durPaso_Horas:= -1;
end;

procedure TCronVar.inicializar(nCronicas, nPasos: integer; dtIni: TDateTime;
  durPaso_Horas: NReal);
var
  k: integer;
begin
  if (self.nCronicas = -1) and (self.nPasos = -1) then
  begin
    if ((nCronicas <= 0) or
        (nPasos <= 0)) then
      raise Exception.Create('TCronVar.inicializar: llamaron al inicializar de la CronVar ' + nombre + ' con nCronicas o nPasos <= 0. Asegurese que todas las CronVars que son usadas como parmetro en una PostOper sean utilizadas antes por una CronOper.');

    self.nPasos:= nPasos;
    self.nCronicas:= nCronicas;
    self.dtIni:= dtIni;
    self.DurPaso_horas:= DurPaso_Horas;

    setlength( v, nPasos );
    for k:= 0 to nPasos - 1 do
      setlength( v[k], NCronicas );
  end
  else if (self.nCronicas <> nCronicas) or (self.nPasos <> nPasos) then
  begin
    raise Exception.Create('TCronVar.inicializar: trate de cambiarle el tamao a la CronVar ya inicializada '
                           + nombre + ' oldNCronicas = ' + IntToStr(self.nCronicas) +
                           ', nCronicas = ' + IntToStr(nCronicas) + ', oldNPasos = '
                           + IntToStr(self.nPasos) + ', nPasos = ' + IntToStr(nPasos));
  end;
end;

function TCronVar.vpe(kPaso: Integer; probabildadExcedencia: NReal): NReal;
var
  i: Integer;
  arrayAux: TDAofNReal;
  res: NReal;
begin
  SetLength(arrayAux, Length(v[kPaso]));
  for i:= 0 to high(v[kPaso]) do
    arrayAux[i]:= v[kPaso][i];
  QuickSort_Decreciente(arrayAux);
  res:= arrayAux[trunc(probabildadExcedencia * high(arrayAux))];
  SetLength(arrayAux, 0);
  result:= res;
end;

procedure TCronVar.Free;
var
  k: integer;
begin
  for k:= 0 to nPasos - 1 do
    setlength( v, 0 );
  setlength( v, 0 );
  inherited Free;
end;

function TCronVar.serialize: string;
begin
  result:= encomille(nombre)
end;


//--------------------
//Mtodos de TCronOper
//====================

function TCronOper.referenciaCronVar(cronVar: TCronVar): boolean;
begin
  result:= Self.res = cronVar;
end;


function TCronOper.resultados: String;
begin
  result:= res.nombre;
end;

function TCronOper.parametrosAdicionales: String;
begin
  result:= '-';
end;


function TCronOper.serialize: string;
begin
  result:=  tipo + #9 + encomille( res.nombre);
end;

//----------------------------
//Mtodos de TCronOperUnIndice
//============================

function TCronOperUnIndice.referenciaIndice(indice: TVarIdxs): boolean;
begin
  result:= Self.param1 = indice;
end;

function TCronOperUnIndice.parametrosIndice: String;
begin
  Result:= param1.nombreIndice;
end;

//------------------------------
//Mtodos de TCronOperDosIndices
//==============================

function TCronOperDosIndices.referenciaIndice(indice: TVarIdxs): boolean;
begin
  result:= inherited referenciaIndice(indice) or (self.param2 = indice);
end;

function TCronOperDosIndices.parametrosIndice: String;
begin
  if param2 <> NIL then  
    Result:= inherited parametrosIndice + ', ' + param2.nombreIndice
  else
    result:= inherited parametrosIndice;
end;

//-------------------------------
//Mtodos de TCronOperMultiIndice
//===============================

function TCronOperMultiIndice.referenciaIndice(indice: TVarIdxs): boolean;
var
  res: boolean;
  i: Integer;
begin
  res:= false;
  i:= 0;
  while (i <= high(params)) and not res do
  begin
    res:= params[i] = indice;
    i:= i + 1;
  end;
  result:= res;    
end;

function TCronOperMultiIndice.parametrosIndice: String;
var
  i: Integer;
  res: String;
begin
  res:= params[0].nombreIndice;
  for i:= 1 to high(params) do
    res:= res + ', ' + params[i].nombreIndice;
  result:= res;
end;



//-------------------------
//Mtodos de TCronOper_suma
//=========================

constructor TCronOper_suma.Create( res: TCronVar; param1: TVarIdxs );
begin
  inherited Create;
  self.res:= res;
  self.param1:= param1;
  self.indices:= NIL;
end;

procedure TCronOper_suma.prepararse;
begin
  res.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, param1.simRes.durPaso_);
  self.indices:= param1.idxs;
end;

procedure TCronOper_suma.evaluar;
begin
  res.v[iPaso][iCronica]:= param1.simRes.suma(indices);
end;

class function TCronOper_suma.ayudaCronOper: String;
begin
  result:= '< suma resCronVar param1Idxs >'#13#10 +
           'Suma los valores seleccionados por los ndices (param1Idxs) y el resultado se'#13#10 +
           'guarda en (resCronVar).'#13#10;
end;

class function TCronOper_suma.tipo: String;
begin
  result:= 'suma';
end;


function TCronOper_suma.serialize: string;
begin
  result:= inherited serialize+ #9 +
          encomille(param1.nombreIndice);
end;

//---------------------------------
//Mtodos de TCronOper_sumaConSigno
//=================================

constructor TCronOper_sumaConSigno.Create( resPos, resNeg: TCronVar; param1: TVarIdxs );
begin
  inherited Create;
  self.res:= resPos;
  self.resNeg:= resNeg;
  self.param1:= param1;
  self.indices:= NIL;
end;

procedure TCronOper_sumaConSigno.prepararse;
begin
  res.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, param1.simRes.durPaso_);
  resNeg.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, param1.simRes.durPaso_);
  self.indices:= param1.idxs;
end;

procedure TCronOper_sumaConSigno.evaluar;
begin
  param1.simRes.sumaConSigno(indices, sumaPos, sumaNeg);
  res.v[iPaso][iCronica]:= sumaPos;
  resNeg.v[iPaso][iCronica]:= sumaNeg;
end;

class function TCronOper_sumaConSigno.ayudaCronOper: String;
begin
  result:= '< sumaConSigno resPosCronVar resNegCronVar param1Idxs >'#13#10 +
           'Suma los valores seleccionados por los ndices (param1Idxs) y el resultado lo'#13#10 +
           'guarda en (resPosCronVar) o (resNegCronVar) segn sea positivo o negativo.'#13#10 +
           'Se pens para analizar el flujo por una interconexin entre pases y poder as'#13#10 +
           'tener por separado flujo de importacin y de exportacin.'#13#10;
end;

class function TCronOper_sumaConSigno.tipo: String;
begin
  result:= 'sumaConSigno';
end;

function TCronOper_sumaConSigno.serialize: string;
begin
   result:= inherited serialize+ #9 +
          encomille(resNeg.nombre) + #9 +
          encomille(param1.nombreIndice);
end;

function TCronOper_sumaConSigno.referenciaCronVar(cronVar: TCronVar): boolean;
begin
  result:= inherited referenciaCronVar(cronVar) or (cronVar = Self.resNeg);
end;

function TCronOper_sumaConSigno.resultados: String;
begin
  result:= inherited resultados + ', ' + resNeg.nombre;
end;

//-----------------------------
//Mtodos de TCronOper_Combinar
//=============================

constructor TCronOper_Combinar.Create( res: TCronVar; params: TDAOfTVarIdxs; coefs: TDAofNReal );
begin
  if length(coefs) <> Length(params) then
    raise Exception.Create('TCronOper_Combinar.Create: la cantidad de coeficientes pasados como parmetro no coincide con la cantidad de indices');
  inherited Create;
  self.res:= res;
  self.coefs:= coefs;
  self.params:= params;
  self.indices:= NIL;
end;

procedure TCronOper_Combinar.prepararse;
var
  i: Integer;
begin
  for i:= 1 to high(params) do
    if params[0].simRes <> params[i].simRes then
      raise Exception.Create('TCronOper_Combinar.Create: los indices pasados como parametro no corresponden al mismo SimRes');
  res.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, params[0].simRes.durPaso_);

  SetLength(indices, Length(params));
  for i:= 0 to high(params) do
    self.indices[i]:= params[i].idxs;
end;


procedure TCronOper_Combinar.Evaluar;
begin

  res.v[iPaso][iCronica]:= params[0].simRes.combinar(indices, coefs);
end;

class function TCronOper_Combinar.ayudaCronOper: String;
begin
  result:= '< combinar resCronVar nIndices param1Idxs ... paramNIndicesIdxs coef1Real ... coefnIndicesReal >'#13#10 +
           'Realiza la combinacin lineal de los coeficientes por los valores seleccionados por los indices'#13#10 +
           'coef1Real * param1Idxs + ... + coefnIndicesReal * paramNIndicesIdxs y el resultado lo'#13#10 +
           'guarda en resCronVar. nIndices es la cantidad de indices (y coeficientes) que se van a usar.'#13#10;
end;

class function TCronOper_Combinar.tipo: String;
begin
  result:= 'combinar';
end;

procedure TCronOper_Combinar.Free;
begin
  SetLength(indices, 0);
  inherited Free;
end;

function TCronOper_Combinar.parametrosAdicionales: String;
begin
  result:= 'coeficientes= ' + TDAOfNRealToStringSinTamanio(coefs, 16, 10, ', ');
end;

function TCronOper_Combinar.serialize: string;
begin
    result:= inherited serialize + #9 +
          TDAOfNRealToTabbedStringConTamanio( coefs, 8, 3) + #9 +
          TDAOfTVarIdxsToString( params);
end;

//-----------------------------
//Mtodos de TCronOper_promedio
//=============================

constructor TCronOper_promedio.Create( res: TCronVar; param1: TVarIdxs );
begin
  inherited Create;
  self.res:= res;
  self.param1:= param1;
  self.indices:= NIL;
end;

procedure TCronOper_promedio.prepararse;
begin
  res.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, param1.simRes.durPaso_);
  self.indices:= param1.idxs;
end;


procedure TCronOper_promedio.evaluar;
begin
  res.v[iPaso][iCronica]:= param1.simRes.promedio(indices);
end;

class function TCronOper_promedio.ayudaCronOper: String;
begin
  result:= '< promedio resCronVar param1Idxs >'#13#10 +
           'Caclula el promedio de los valores seleccionados por (param1Indxs) y lo guarda'#13#10 +
           'en (resCronVar)'#13#10;
end;

class function TCronOper_promedio.tipo: String;
begin
  result:= 'promedio';
end;


function TCronOper_promedio.serialize: String;
begin
  result:= inherited serialize +#9 +
      encomille( param1.nombreIndice );
end;

//------------------------------------------
//Mtodos de TCronOper_sumaProductoConDurpos
//==========================================

constructor TCronOper_sumaProductoConDurpos.Create( res: TCronVar; param1: TVarIdxs );
begin
  inherited Create;
  self.res:= res;
  self.param1:= param1;
  self.indices:= NIL;
end;

procedure TCronOper_sumaProductoConDurpos.prepararse;
begin
  res.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, param1.simRes.durPaso_);
  self.indices:= param1.idxs;
end;

procedure TCronOper_sumaProductoConDurpos.evaluar;
begin
  res.v[iPaso][iCronica]:= param1.simRes.sumaproductocondurpos(indices);
end;

class function TCronOper_sumaProductoConDurpos.ayudaCronOper: String;
begin
  result:= '< sumaProductoConDurpos resCronVar param1Idxs >'#13#10;
end;

class function TCronOper_sumaProductoConDurpos.tipo: String;
begin
  result:= 'sumaProductoConDurpos';
end;

function TCronOper_sumaProductoConDurpos.serialize: String;
begin
  result:= inherited serialize + #9 + encomille(param1.nombreIndice);
end;


//-----------------------------------------------
//Mtodos de TCronOper_sumaProductoConDurposHasta
//===============================================

constructor TCronOper_sumaProductoConDurposHasta.Create( res: TCronVar; param1: TVarIdxs; kposteHasta: integer );
begin
  inherited Create;
  self.res:= res;
  self.param1:= param1;
  self.kposteHasta:= kposteHasta;
  self.indices:= NIL;
end;

procedure TCronOper_sumaProductoConDurposHasta.prepararse;
begin
  res.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, param1.simRes.durPaso_);
  self.indices:= param1.idxs;
end;

procedure TCronOper_sumaProductoConDurposHasta.evaluar;
begin
  res.v[iPaso][iCronica]:= param1.simRes.sumaproductoconDurposHasta(indices, kposteHasta);
end;

class function TCronOper_sumaProductoConDurposHasta.ayudaCronOper: String;
begin
  result:= '< sumaProductoConDurposHasta resCronVar param1Idxs kposteHasta_integer >'#13#10;
end;

class function TCronOper_sumaProductoConDurposHasta.tipo: String;
begin
  result:= 'sumaProductoConDurposHasta';
end;

function TCronOper_sumaProductoConDurposHasta.parametrosAdicionales: String;
begin
  result:= 'kPosteHasta= ' + IntToStr(kposteHasta);
end;



function TCronOper_sumaProductoConDurposHasta.serialize: String;
begin
  result:= inherited serialize  + #9 + encomille(param1.nombreIndice)
            + #9 +  IntToStr( kposteHasta );
end;

//------------------------------------------
//Mtodos de TCronOper_promedioPonderadoPorDurpos
//==========================================

constructor TCronOper_promedioPonderadoPorDurpos.Create( res: TCronVar; param1: TVarIdxs );
begin
  inherited Create;
  self.res:= res;
  self.param1:= param1;
  self.indices:= NIL;
end;

procedure TCronOper_promedioPonderadoPorDurpos.prepararse;
begin
  res.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, param1.simRes.durPaso_);
  self.indices:= param1.idxs;
end;

procedure TCronOper_promedioPonderadoPorDurpos.evaluar;
begin
  res.v[iPaso][iCronica]:= param1.simRes.promedioPonderadoPorDurpos(indices);
end;

class function TCronOper_promedioPonderadoPorDurpos.ayudaCronOper: String;
begin
  result:= '< promedioPonderadoPorDurpos resCronVar param1Idxs >'#13#10;
end;

class function TCronOper_promedioPonderadoPorDurpos.tipo: String;
begin
  result:= 'promedioPonderadoPorDurpos';
end;

function TCronOper_promedioPonderadoPorDurpos.serialize: String;
begin
  result:= inherited serialize + #9 + encomille(param1.nombreIndice);
end;

//-----------------------------------
//Mtodos de TCronOper_filtrarCronica
//===================================

constructor TCronOper_filtrarCronica.Create( res: TCronVar; param1: TVarIdxs; kCronica: Integer);
begin
  inherited Create;
  self.res:= res;
  self.param1:= param1;
  self.kCronica:= kCronica;
  self.indices:= NIL;
end;

procedure TCronOper_filtrarCronica.prepararse;
begin
  res.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, param1.simRes.durPaso_);
  self.indices:= param1.idxs;
end;

procedure TCronOper_filtrarCronica.Evaluar;
begin
  if iCronica = kCronica - 1 then
    res.v[iPaso][iCronica]:= param1.simRes.promedio(indices)
  else
    res.v[iPaso][iCronica]:= 0;
end;

class function TCronOper_filtrarCronica.ayudaCronOper: String;
begin
  result:= '< filtrarCronica resCronVar param1Idxs kCronica_integer >'#13#10;
end;

class function TCronOper_filtrarCronica.tipo: String;
begin
  result:= 'filtrarCronica';
end;

function TCronOper_filtrarCronica.parametrosAdicionales: String;
begin
  result:= 'Crnica= ' + IntToStr(kCronica);
end;

function TCronOper_filtrarCronica.serialize: String;
begin
  result:= inherited serialize + #9 +
          encomille(param1.nombreIndice) + #9 +
          IntToStr(kCronica);
end;


//-------------------------------------------------
//Mtodos de TCronOper_sumaProductoConDurposTopeado
//=================================================

constructor TCronOper_sumaProductoConDurposTopeado.Create( resSP, resRecorte: TCronVar; param1: TVarIdxs; TopeDe1: NReal );
begin
  inherited Create;
  self.res:= resSP;
  self.resRecorte:= resRecorte;
  self.param1:= param1;
  self.TopeDe1:= TopeDe1;
  self.indices:= NIL;
end;

procedure TCronOper_sumaProductoConDurposTopeado.prepararse;
begin
  res.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, param1.simRes.durPaso_);
  if resRecorte <> nil then
    resRecorte.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, param1.simRes.durPaso_);
  self.indices:= param1.idxs;
end;

procedure TCronOper_sumaProductoConDurposTopeado.Evaluar;
begin
  res.v[iPaso][iCronica]:= param1.simRes.sumaProductoConDurposTopeado(indices, TopeDe1, recorte);
  if resRecorte <> nil then
    resRecorte.v[iPaso][iCronica]:= recorte;
end;

class function TCronOper_sumaProductoConDurposTopeado.ayudaCronOper: String;
begin
  result:= '< sumaProductoConDurposTopeado resSPCronVar resRecorteCronVar paramAIdxs TopeDeA_NReal>'#13#10 +
           'Calcula la suma producto de los valores seleccionados por paramAIdxs, considerando topeados'#13#10 +
           'los valores de A por el valor del parmetro TopeDeA.'#13#10 +
           'Los resultados e cargan en resSPCronVar y los valores recortados se cargan en resRecorteCronVar.'#13#10 +
           'Si no interesa asignar los recortes se puede poner "nil" en resRecorteCronVar.'#13#10;
end;

class function TCronOper_sumaProductoConDurposTopeado.tipo: String;
begin
  result:= 'sumaProductoConDurposTopeado';
end;

function TCronOper_sumaProductoConDurposTopeado.referenciaCronVar(cronVar: TCronVar): boolean;
begin
  result:= inherited referenciaCronVar(cronVar) or (Self.resRecorte = cronVar);
end;

function TCronOper_sumaProductoConDurposTopeado.resultados: String;
begin
  if resRecorte <> NIL then
    result:= inherited resultados + ', ' + resRecorte.nombre
  else
    result:= inherited resultados;
end;

function TCronOper_sumaProductoConDurposTopeado.parametrosAdicionales: String;
begin
  result:= 'TopeDe1= ' + FloatToStrF(TopeDe1, ffGeneral, 16, 10);
end;


function TCronOper_sumaProductoConDurposTopeado.serialize: String;
begin
  result:= inherited serialize + #9 +
          encomille(nombreCronVar(resRecorte)) + #9 +
          encomille(param1.nombreIndice) + #9 +
          FloatToStr(TopeDe1);

end;

//------------------------------------------------------
//Mtodos de TCronOper_sumaDobleProductoConDurposTopeado
//======================================================

constructor TCronOper_sumaDobleProductoConDurposTopeado.Create( resSP, resRecorte: TCronVar; param1, param2: TVarIdxs; TopeDe2: NReal );
begin
  inherited Create;
  self.res:= resSP;
  self.resRecorte:= resRecorte;
  self.param1:= param1;
  self.param2:= param2;
  self.TopeDe2:= TopeDe2;
  self.indices:= NIL;
  self.indices2:= NIL;
end;

procedure TCronOper_sumaDobleProductoConDurposTopeado.prepararse;
begin
  Assert(param1.simRes = param2.simRes);
  //Se asume que paramA y paramB son indices sobre el mismo simres
  res.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, param1.simRes.durPaso_);
  if resRecorte <> nil then
    resRecorte.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, param1.simRes.durPaso_);
  self.indices:= param1.idxs;
  self.indices2:= param2.idxs;
end;

procedure TCronOper_sumaDobleProductoConDurposTopeado.evaluar;
begin
  res.v[iPaso][iCronica]:= param1.simRes.sumaDobleProductoConDurposTopeado(indices, indices2, TopeDe2, recorte);
  if resRecorte <> nil then
    resRecorte.v[iPaso][iCronica]:= recorte;
end;

class function TCronOper_sumaDobleProductoConDurposTopeado.ayudaCronOper: String;
begin
  result:= '< sumaDobleProductoConDurposTopeado resSPCronVar resRecorteCronVar paramAIdxs paramBIdxs TopeDeB_NReal>'#13#10 +
           'Calcula la suma producto de los valores seleccionados por paramAIdxs y paramBIdxs, considerando topeados'#13#10 +
           'los valores B por el valor del parmetro TopeDeB.'#13#10 +
           'Los resultados e cargan en resSPCronVar y los valores recortados se cargan en resRecorteCronVar.'#13#10 +
           'Si no interesa asignar los recortes se puede poner "nil" en resRecorteCronVar.'#13#10 +
           'Esta operacin es til para calcular remuneraciones al spot. Para ello poner en paramBIdxs los indices'#13#10 +
           'que seleccionan los costos marginales de un nodo y en TopeDeB_NReal el tope puesto al precio spot.'#13#10;
end;

class function TCronOper_sumaDobleProductoConDurposTopeado.tipo: String;
begin
  result:= 'sumaDobleProductoConDurposTopeado';
end;

function TCronOper_sumaDobleProductoConDurposTopeado.referenciaCronVar(cronVar: TCronVar): boolean;
begin
  result:= inherited referenciaCronVar(cronVar) or (self.resRecorte = cronVar);
end;

function TCronOper_sumaDobleProductoConDurposTopeado.resultados: String;
begin
  if resRecorte <> NIL then
    result:= inherited resultados + ', ' + resRecorte.nombre
  else
    result:= inherited resultados;
end;

function TCronOper_sumaDobleProductoConDurposTopeado.parametrosAdicionales: String;
begin
  result:= 'TopeDe2= ' + FloatToStrF(TopeDe2, ffGeneral, 16, 10);
end;

function TCronOper_sumaDobleProductoConDurposTopeado.serialize: String;
begin
  result:= inherited serialize + #9 +
            encomille(nombreCronVar(resRecorte)) + #9 +
          encomille(param1.nombreIndice) + #9 +
          encomille(param2.nombreIndice) + #9 +
          FloatToStr(TopeDe2);

end;


//---------------------------
//Mtodos de TCronOper_suma_m
//===========================

constructor TCronOper_suma_m.Create( res: TCronVar; params: TDAOfTVarIdxs );
begin
  inherited Create;
  self.res:= res;
  self.params:= params;
  self.indices:= NIL;
end;

procedure TCronOper_suma_m.prepararse;
var
  i: Integer;
begin
  //Se asume que todos los params son indices sobre el mismo simRes
  res.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, params[0].simRes.durPaso_);
  SetLength(self.indices, length(params));
  for i:= 0 to high(params) do
    indices[i]:= params[i].idxs;
end;

procedure TCronOper_suma_m.evaluar;
begin
  res.v[iPaso][iCronica]:= params[0].simRes.suma_m(indices);
end;

class function TCronOper_suma_m.ayudaCronOper: String;
begin
  result:= '<suma_m resCronVar; params: TDAOfTVarIdxs )'#13#10 +
           'Esta operacin realiza la suma de un conjunto de variables especificadas como ndices.'#13#10 +
           'el primer parmetro tiene que ser un entero indicando la cantidad de ndices que se sumarn.'#13#10 +
           'el resto de los parmetros son los nombres de cada uno de los ndices a sumar.'#13#10 +
           'Por Ejemplo, si los ndices iA_GWh e iB_GWh seleccionan las energias generadas por la central'#13#10 +
           'A y la central B respectivamente en todos sus postes, la siguiente definicion guardara'#13#10 +
           'la energa generada por las dos centrales en la variabel sumE1E2 que tendra que definirse en (+CronVar)'#13#10 +
           'suma_m sumE1E2 2 iA_GWh iB_GWh'#13#10;
end;

class function TCronOper_suma_m.tipo: String;
begin
  result:= 'suma_m';
end;

procedure TCronOper_suma_m.Free;
begin
  SetLength(indices, 0);
  inherited Free;
end;

function TCronOper_suma_m.serialize: String;
begin
  result:= inherited serialize + #9 +
            TDAOfTVarIdxsToStringConTamanio( params );
end;



//-------------------------------
//Mtodos de TCronOper_promedio_m
//===============================

constructor TCronOper_promedio_m.Create( res: TCronVar; params: TDAOfTVarIdxs );
begin
  inherited Create;
  self.res:= res;
  self.params:= params;
  self.indices:= NIL;
end;

procedure TCronOper_promedio_m.prepararse;
var
  i: Integer;
begin
  //Se asume que todos los params son indices sobre el mismo simRes
  res.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, params[0].simRes.durPaso_);
  SetLength(self.indices, length(params));
  for i:= 0 to high(params) do
    indices[i]:= params[i].idxs;
end;

procedure TCronOper_promedio_m.evaluar;
begin
  res.v[iPaso][iCronica]:= params[0].simRes.promedio_m(indices);
end;

class function TCronOper_promedio_m.ayudaCronOper: String;
begin
  result:= '< promedio_m( res: TCronVar; params: TDAOfTVarIdxs ) >'#13#10;
end;

class function TCronOper_promedio_m.tipo: String;
begin
  result:= 'promedio_m';
end;

procedure TCronOper_promedio_m.Free;
begin
  SetLength(indices, 0);
  inherited Free;
end;

function TCronOper_promedio_m.serialize: String;
begin
  result:= inherited serialize + #9 +
          TDAOfTVarIdxsToStringConTamanio(params);  
end;

//--------------------------------------------
//Mtodos de TCronOper_sumaProductoConDurpos_m
//============================================

constructor TCronOper_sumaProductoConDurpos_m.Create( res: TCronVar; params: TDAOfTVarIdxs );
begin
  inherited Create;
  self.res:= res;
  self.params:= params;
  self.indices:= NIL;
end;

procedure TCronOper_sumaProductoConDurpos_m.prepararse;
var
  i: Integer;
begin
  //Se asume que todos los params son indices sobre el mismo simRes
  res.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, params[0].simRes.durPaso_);
  SetLength(self.indices, length(params));
  for i:= 0 to high(params) do
    indices[i]:= params[i].idxs;
end;

procedure TCronOper_sumaProductoConDurpos_m.evaluar;
begin
  res.v[iPaso][iCronica]:= params[0].simRes.sumaproductocondurpos_m(indices);
end;

class function TCronOper_sumaProductoConDurpos_m.ayudaCronOper: String;
begin
  result:= '< sumaProductoConDurpos_m( res: TCronVar; params: TDAOfTVarIdxs ) >'#13#10;
end;

class function TCronOper_sumaProductoConDurpos_m.tipo: String;
begin
  result:= 'sumaProductoConDurpos_m';
end;

procedure TCronOper_sumaProductoConDurpos_m.Free;
begin
  SetLength(indices, 0);
  inherited Free;
end;

function TCronOper_sumaProductoConDurpos_m.serialize: String;
begin
  result:= inherited serialize + #9 +
          TDAOfTVarIdxsToStringConTamanio(params);  
end;


//-----------------------------
//Mtodos de TCronOper_Maximo_m
//=============================

constructor TCronOper_Maximo_m.Create( res: TCronVar; params: TDAOfTVarIdxs );
begin
  inherited Create;
  self.res:= res;
  self.params:= params;
  self.indices:= NIL;
end;

procedure TCronOper_Maximo_m.prepararse;
var
  i: Integer;
begin
  //Se asume que todos los params son indices sobre el mismo simRes
  res.inicializar(uSimResGlobs.nCronicasDelEstudio, uSimResGlobs.nPasosDelEstudio, uSimResGlobs.desde, params[0].simRes.durPaso_);
  SetLength(self.indices, length(params));
  for i:= 0 to high(params) do
    indices[i]:= params[i].idxs;
end;

procedure TCronOper_Maximo_m.evaluar;
begin
  res.v[iPaso][iCronica]:= params[0].simRes.maximo_m(indices);
end;

class function TCronOper_Maximo_m.ayudaCronOper: String;
begin
  result:= '<maximo_m resCronVar; params: TDAOfTVarIdxs )'#13#10 +
           'Esta operacin obtiene el mximo valor de un conjunto de variables especificadas como ndices.'#13#10 +
           'El primero pmetro es la cronVar resultado.'#13#10+
           'El segundo parmetro tiene que ser un entero indicando la cantidad de ndices en que se buscar el mximo.'#13#10 +
           'el resto de los parmetros son los nombres de cada uno de los ndices a sumar.'#13#10 +
           'Por Ejemplo, si los ndices iA_GWh e iB_GWh seleccionan las energias generadas por la central'#13#10 +
           'A y la central B respectivamente en todos sus postes, la siguiente definicion guardara'#13#10 +
           'la mxima energa generada entre las dos centrales en la variabel maxE1E2 que tendra que definirse en (+CronVar)'#13#10 +
           'maximo_m maxE1E2 2 iA_GWh iB_GWh'#13#10;
end;

class function TCronOper_Maximo_m.tipo: String;
begin
  result:= 'maximo_m';
end;

procedure TCronOper_Maximo_m.Free;
begin
  SetLength(indices, 0);
  inherited Free;
end;

function TCronOper_Maximo_m.serialize: String;
begin
  result:= inherited serialize + #9 +
            TDAOfTVarIdxsToStringConTamanio( params );
end;


end.
